function [ data, properties ] = load_bbdata( bbfile, start_time, duration )
% load_bbdata() loads VFBB data that matches the specified start time and duration.
%
% [ data, properties ] = load_bbdata( bbfile, [ year day hour minute second ], seconds )
%
% The output data vector is initialized with NaNs.  Data from bbfile that fit
% within the specified start time and duration are loaded into the data vector.
% The bbfile properties structure includes .t0, the start time vector extended to
% microseconds, .dt, the sample interval, in seconds, and .srate, the sample rate,
% in samples per second.
%
% To load additional VFBB data into the same data vector, call merge_bbdata().
%

%
% Author:  Lawrence M. Baker
%          U.S. Geological Survey
%          345 Middlefield Road  MS977
%          Menlo Park, CA  94025
%          baker@usgs.gov
%
%                                 Disclaimer
%
% Although  this  program  has  been  used by the U.S. Geological Survey, no
% warranty, expressed or implied, is made by the USGS as to the accuracy and
% functioning  of  the  program  and related program material, nor shall the
% fact of distribution constitute any such warranty, and  no  responsibility
% is assumed by the USGS in connection therewith.
%
%
% Modification History:
%
%  9-Feb-2005  L. M. Baker      Original version.
% 10-Feb-2005  L. M. Baker      Add warnings if orientations don't match.
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Exactly 3 input arguments
if ( nargin ~= 3 )
	error( 'load_bbdata requires 3 input arguments; type help load_bbdata.' );
end

% Exactly 2 output arguments
if ( nargout ~= 2 )
	error( 'load_bbdata returns 2 output results; type help load_bbdata.' );
end

% Verify start_time
year   = start_time(1);
day    = start_time(2);
hour   = start_time(3);
minute = start_time(4);
second = start_time(5);
if ( year >= 0 & year <= 99 )
	error( 'Invalid start_time; use 4-digit year.' );
end
if ( year   < 0 | ...
     day    < 1 | day    > 365 + leap_days( year) | ...
	 hour   < 0 | hour   >  23 | ...
	 minute < 0 | minute >  59 | ...
	 second < 0 | second >  59 ) % We don't handle leap seconds
	error( 'Invalid start_time; must be [ year day hour minute second ].' );
end

% Verify seconds duration
if ( duration <= 0 )
	error(' Invalid duration; must be > 0.' );
end

% Read bbfile integer and real headers
ihdr = bbihdr( bbfile );
rhdr = bbrhdr( bbfile );
inull = ihdr(3);
rnull = rhdr(2);

vertical   = ihdr(41);
horizontal = ihdr(42);
if ( ( vertical == inull ) | ( horizontal == inull ) )
	orientation = 'unknown';
else
	switch ( mod( vertical, 360 ) )
		case { 0, 180 }
			orientation = 'vertical (positive ';
			if ( vertical == 0 )
				orientation = [ orientation 'up)' ];
			else
				orientation = [ orientation 'down)' ];
			end
		case  90
			orientation = 'horizontal (positive ';
			switch ( mod( horizontal, 360 ) )
				case   0
					orientation = [ orientation 'N)' ];
				case  90
					orientation = [ orientation 'E)' ];
				case 180
					orientation = [ orientation 'S)' ];
				case 270
					orientation = [ orientation 'W)' ];
				otherwise
					orientation = [ orientation int2str( horizontal ) ...
									' degrees clockwise from N)' ];
			end
		otherwise
			orientation = [ 'positive ' int2str( vertical ) ...
							' degrees down from vertical, ' ...
							int2str( horizontal ) ...
							' degrees clockwise from N' ];
	end
end
motion = ihdr(254);
switch ( motion )
	case 1
		units = 'acceleration (cm/s/s)';
	case 2
		units = 'velocity (cm/s)';
	case 3
		units = 'displacement (cm)';
	case 50
		units = 'volumetric strain';
	otherwise
		units = 'unknown units';
end
samples_per_second = rhdr(5);

disp( [ 'VFBB data is ' orientation ' ' units ', ' ...
		num2str( samples_per_second ) ' samples per second.' ] );

% Allocate the output data vector and fill it with NaNs
no_of_samples = duration * samples_per_second;
data = NaN * ones( 1, no_of_samples );

% Save the start time to microseconds, the sample interval, the sample rate,
% and the orientation
properties.t0 = [ start_time 0 0 ];
properties.dt = 1 / samples_per_second;
properties.srate = samples_per_second;
properties.vertical   = vertical;
properties.horizontal = horizontal;

% Call merge_bbdata() to load the bbfile data
data = merge_bbdata( bbfile, data, properties );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function days = leap_days( year )

days = ( rem( year,   4 ) == 0 ) & ...
	   ( rem( year, 100 ) ~= 0 | rem( year, 400 ) == 0 );
